# wsse/client/requests/auth.py
# coding=utf-8
# py-wsse
# Authors: Rushy Panchal, Naphat Sanguansin, Adam Libresco, Jérémie Lumbroso
# Date: September 1st, 2016
# Description: Authentication handler for requests library.

try:
	import requests
except ImportError:
	raise ImportError('requests is required to use the requests plugin')

from ... import settings, utils

class WSSEAuth(requests.auth.AuthBase):
	'''
	Attaches HTTP Digest Authentication to the given Request object.

	Usage:
		requests.get(url, auth=WSSEAuth(username, password))

	The auth object isreusable, and can be defined separately if several
	requests will be made with the same username & password pair.

	:param username: username to include in request
	:type username: str

	:param password: password to authenticate with
	:type password: str
	'''
	def __init__(self, username, password):
		self.token_builder = utils.TokenBuilder(username, password)

	def __call__(self, r):
		'''Attach the appropriate request header to attempt WSSE authentication.'''
		token = self.token_builder.make_token()
		r.headers[settings.REQUEST_HEADER] = token
		return r



# wsse/server/django/models.py
# coding=utf-8
# py-wsse
# Authors: Rushy Panchal, Naphat Sanguansin, Adam Libresco, Jérémie Lumbroso
# Date: September 1st, 2016
# Description: Database models for storing the nonces and user secrets.

import datetime

from django.db import models
from django.contrib.auth.models import User

from ... import settings, utils

def _generate_secret():
	'''
	Generate a new secret key.
	'''
	return utils._random_string(length = settings.SECRET_KEY_LENGTH)

class WSSEEvent(models.Model):
	'''
	The `WSSEEvent` object records an event that occured with the WSSE
	authentication. Specifically, it records when a nonce was used to prevent
	immediate re-use. 
	'''
	nonce = models.CharField(max_length = settings.NONCE_LENGTH, db_index = True)

	# We cannot use `auto_now_add = True` here because the timezone for that
	# is not explicitly known. Instead, we default to using the current time in
	# UTC, which guarantees the timezone, especially because all the timestamps
	# are stored in UTC.
	timestamp = models.DateTimeField(default = datetime.datetime.utcnow,
		db_index = True)

class UserSecret(models.Model):
	'''
	The `UserSecret` object connects every user to their secret key. The secret
	key can be deleted to prevent access without affecting the user.
	'''
	user = models.OneToOneField(User, on_delete = models.CASCADE,
		db_index = True)
	secret = models.CharField(max_length = settings.SECRET_KEY_LENGTH,
		default = _generate_secret, blank = True, null = False)



# wsse/server/drf/authentication.py
# coding=utf-8
# py-wsse
# Authors: Rushy Panchal, Naphat Sanguansin, Adam Libresco, Jérémie Lumbroso
# Date: September 1st, 2016
# Description: Django REST Framework Authentication backend.

try:
	from django.contrib.auth.models import User
	from rest_framework import authentication
	from rest_framework.exceptions import AuthenticationFailed
except ImportError:
	raise ImportError('django and rest_framework required to use DRF plugin')

from ... import utils, exceptions, settings
from ..django.models import UserSecret

class WSSEAuthentication(authentication.BaseAuthentication):
	'''
	The `WSSEAuthentication` backend provides WSSE authentication functionality
	to DRF. It can applied as an authentication backend or used on a
	per-view basis.
	'''
	def _get_password(self, username):
		'''
		Get the password for a given username.

		:param username: username to get password for
		:type username: str

		:return: password for user
		:rtype: str

		:raises KeyError: user not found
		'''
		try:
			user_secret = UserSecret.objects.get(user__username = username)
		except UserSecret.DoesNotExist:
			raise KeyError('User not found')
	
		return user_secret.secret

	def authenticate(self, request):
		'''
		Authenticate a request.

		:param request: request to authenticate
		:type request: rest_framework.request.Request

		:return: (user, auth) if authentication succeeds, None if not attempted
		:rtype: tuple

		:raises rest_framework.exceptions.AuthenticationFailed:
			if authentication fails
		'''
		header_name = utils._django_header(settings.REQUEST_HEADER)
		wsse_header = request.META.get(header_name)
		if not wsse_header:
			# If no header is provided, do not attempt authentication.
			return None

		try:
			username = utils.check_token(wsse_header, self._get_password)
		except exceptions.WSSEException as e:
			raise AuthenticationFailed(str(e))

		# There is no need to error-check getting the user because that is
		# already done in _get_password.
		if not username:
			raise AuthenticationFailed('User could not be authenticated')

		user = User.objects.get(username = username)
		return (user, None)

	def authenticate_header(self, request):
		'''
		Return the authentication header.

		:param request: incoming request
		:type request: rest_framework.request.Request
		'''
		return 'WSSE realm="", profile="UsernameToken"'



# wsse/exceptions.py
# coding=utf-8
# py-wsse
# Authors: Rushy Panchal, Naphat Sanguansin, Adam Libresco, Jérémie Lumbroso
# Date: August 30th, 2016
# Description: Exceptions for py-wsse.

class WSSEException(Exception):
	'''
	Base class for all WSSE exceptions.
	'''
	pass

class AlgorithmNotSupported(WSSEException):
	'''
	Raised when a suitable digest algorithm is not found.
	'''
	pass

class AlgorithmProhibited(WSSEException):
	'''
	Raised when an algorithm is prohibited.
	'''
	pass

class InvalidToken(WSSEException):
	'''
	Raised when a token is invalid.
	'''
	pass

class InvalidNonce(WSSEException):
	'''
	Raised when a nonce is invalid.
	'''
	pass

class InvalidTimestamp(WSSEException):
	'''
	Raised when a timestamp is invalid.
	'''
	pass

class UserException(WSSEException):
	'''
	Raised when a user-related error occurs.
	'''
	pass



# wsse/settings.py
# coding=utf-8
# py-wsse
# Authors: Rushy Panchal, Naphat Sanguansin, Adam Libresco, Jérémie Lumbroso
# Date: August 30th, 2016
# Description: Settings for py-wsse. These settings are shared by the server
# 	and client and can be modified to alter their behavior.

# The duration for which the timestamp is valid (in seconds).
# After that duration, the timestamp will no longer allow authentication.
TIMESTAMP_DURATION = 2 * 60 * 60

# Number of seconds to allow for 'drift' - offsets in the future-timestamp
# checking to allow for minor differences in time settings.
DRIFT_OFFSET = 10

# Timestamp formats. All timestamps are converted to UTC once parsed. If the
# timestamp is naive (i.e does not specify UTC or offset from UTC), it is
# assumed to be UTC.
#
# :note: The %z specifier is only supported on Python 3.2+. A custom parser is
# used when the %z specifier fails.
TIMESTAMP_FORMATS = [
	'%Y-%m-%dT%H:%M:%SZ',    # ISO 8601 in UTC (Z specifies 0 offset)
	'%Y-%m-%dT%H:%M:%S',     # ISO 8601 without timezone - assumed to be UTC
	'%Y-%m-%dT%H:%M:%S.%fZ', # ISO 8601 with microseconds in UTC
	'%Y-%m-%dT%H:%M:%S.%f',  # ISO 8601 with microseconds (assumed UTC)
	'%Y-%m-%dT%H:%M:%S%z',   # ISO 8601 with UTC offset
	'%Y-%m-%dT%H:%M:%S.%f%z' # ISO 8601 with microseconds and UTC offset
	]

# (Maximum) length of the nonce.
# :note: If using a database to store the used nonces, changing this value
# requires a database migration.
NONCE_LENGTH = 64

# Storage backend for the nonces.
NONCE_STORE = 'wsse.server.default.store.SQLiteNonceStore'
NONCE_STORE_ARGS = (':memory:',)

# Name for the request header.
REQUEST_HEADER = 'X-WSSE'

# Whether or not to check the nonce for validity.
SECURITY_CHECK_NONCE = True

# Whether or not to check the timestamp for validity.
SECURITY_CHECK_TIMESTAMP = True

# List of digest algorithms that are allowed. They are checked in this order.
ALLOWED_DIGEST_ALGORITHMS = ['SHA256']

# List of digest algorithms that are prohibited.
PROHIBITED_DIGEST_ALGORITHMS = ['SHA1', 'MD5']

# (Maximum) length of the secret key. This is only enforced when the secrets
# are stored in the database. When changed, a database migration is required.
SECRET_KEY_LENGTH = 64

# Root logger name.
LOGGER_NAME = 'wsse'



# wsse/server/django/__init__.py
# coding=utf-8
# py-wsse
# Authors: Rushy Panchal, Naphat Sanguansin, Adam Libresco, Jérémie Lumbroso
# Date: September 1st, 2016

from django.conf import settings
if not (hasattr(settings, 'TESTING') and settings.TESTING):
	default_app_config = 'wsse.server.django.apps.WsseConfig'



# wsse/utils.py
# coding=utf-8
# py-wsse
# Authors: Rushy Panchal, Naphat Sanguansin, Adam Libresco, Jérémie Lumbroso
# Date: August 30th, 2016
# Description: Shared utility functions for both the WSSE server and client.

import logging
import random
import string
import datetime
import hashlib
import base64
import re
import pydoc

from . import settings
from . import exceptions

try:
	string_types = (str, basestring, unicode)
except NameError:
	string_types = (str,)

logger = logging.getLogger(settings.LOGGER_NAME)

_TOKEN_RE = re.compile(
	r'''
		(?P<key>\w+)\s*=\s* # Key consists of only alphanumerics
		(?P<quote>["']?)    # Optional quote character.
		(?P<value>.*?)      # Value is a non greedy match
		(?P=quote)          # Closing quote equals the first.
		($|,)               # Entry ends with comma or end of string
	''', re.VERBOSE)

_TS_OFFSET_RE = re.compile(
	r'''
		^
		(?P<timestamp>.+)    # Main part of timestamp
		(?P<direction>\+|\-) # Offset direction (+ or -)
		(?P<hour>\d{2})      # Hour offset (HH)
		(?P<minute>\d{2})    # Minute offset (MM)
		$
	''', re.VERBOSE)

_STORES = {}

class TokenBuilder(object):
	'''
	A `TokenBuilder` generates tokens for the given username and password.

	:param username: username for token authentication
	:type username: str

	:param password: password for token authentication
	:type password: str
	'''
	def __init__(self, username, password):
		self.username = username
		self.password = password

	def make_token(self, nonce = None, timestamp = None):
		'''
		Make a new WSSE token with the current username and password.

		:param nonce: nonce to use in token
		:type nonce: str

		:param timestamp: timestamp the nonce was generated at
		:type timestamp: datetime.datetime

		:return: WSSE token
		:rtype: str
		'''
		if not nonce:
			nonce = _random_string()
		if not timestamp:
			timestamp = datetime.datetime.utcnow()

		return make_token(self.username, self.password, nonce, timestamp)

def make_token(username, password, nonce, timestamp, ts_format = None,
	algorithm = None):
	'''
	Make a WSSE token using the provided fields.

	:param username: username for token
	:type username: str

	:param password: password for token
	:type password: str

	:param nonce: nonce for token
	:type nonce: str

	:param timestamp: timestamp the token was generated at
	:type timestamp: datetime.datetime

	:param ts_format: format for the timestamp (optional)
	:type ts_format: str

	:param algorithm: algorithm to use for digest (optional)
	:type algorithm: str

	:return: WSSE token
	:rtype: str
	'''
	if timestamp < (datetime.datetime.utcnow() -
		datetime.timedelta(seconds = settings.TIMESTAMP_DURATION)):
		logger.warning('Timestamp in make_token expired: %s (%ds duration)',
			timestamp, settings.TIMESTAMP_DURATION)

	if not ts_format:
		ts_format = settings.TIMESTAMP_FORMATS[0]
	timestamp_str = timestamp.strftime(ts_format)

	password_digest = _b64_digest(nonce, timestamp_str, password,
		algorithm = algorithm)
	encoded_nonce = base64.b64encode(_to_bytes(nonce))

	fields = (
		('Username', _from_bytes(username)),
		('PasswordDigest', _from_bytes(password_digest)),
		('Nonce', _from_bytes(encoded_nonce)),
		('Created', timestamp_str),
		)

	return ', '.join('{k}="{v}"'.format(k = k, v = v) for k, v in fields)

def check_token(token, get_password = lambda username: username):
	'''
	Check if a token is valid.

	:param token: token to check
	:type token: str

	:param get_password: function to get password given username
	:type get_password: types.FunctionType

	:return: username if check passes or None otherwise
	:rtype: str

	:raises exceptions.InvalidTimestamp: timestamp is in invalid format or
		expired/in future
	:raises exceptions.InvalidNonce: nonce is not proper length or already used
	:raises exceptions.InvalidToken: token is not in proper format
	:raises exceptions.UserException: user's password not found
	:raises exceptions.AlgorithmProhibited: digest algorithm is explicitly
		prohibited
	'''
	username, encoded_digest, encoded_nonce, created = _parse_token(token)

	try:
		nonce = base64.b64decode(encoded_nonce)
	except (TypeError, ValueError) as e:
		msg = 'Nonce should be properly base64 encoded: {}.\nError: {!r}'.format(
			encoded_nonce, e)
		logger.info(msg)
		raise exceptions.InvalidNonce(msg)

	timestamp = _parse_timestamp(created)

	if settings.SECURITY_CHECK_TIMESTAMP:
		now = datetime.datetime.utcnow()
		expired_time = now - datetime.timedelta(
			seconds = settings.TIMESTAMP_DURATION)
		future_time = now + datetime.timedelta(seconds = settings.DRIFT_OFFSET)

		if (expired_time > timestamp or timestamp > future_time):
			msg = '''The timestamp {} is expired or in the future. The timestamp
is required to be between {} and {} - a time window of {} seconds.'''.format(
	created, expired_time.strftime(settings.TIMESTAMP_FORMATS[0]),
	future_time.strftime(settings.TIMESTAMP_FORMATS[0]),
	settings.TIMESTAMP_DURATION)

			logger.info(msg)
			raise exceptions.InvalidTimestamp(msg)

	if settings.SECURITY_CHECK_NONCE:
		if len(nonce) != settings.NONCE_LENGTH:
			msg = 'Nonce should be {} in length, not {}.'.format(
				settings.NONCE_LENGTH, len(nonce))
			logger.info(msg)
			raise exceptions.InvalidNonce(msg)

		nonce_store = _get_nonce_store()

		# Check if the nonce has already been used.
		if nonce_store.has_nonce(nonce):
			msg = '''The nonce {} has already been used. Please use a fresh nonce
with each request.'''.format(nonce)
			logger.info(msg)
			raise exceptions.InvalidNonce(msg)

		nonce_store.add_nonce(nonce)

	try:
		password = get_password(username)
	except KeyError:
		msg = 'User {} not found.'.format(username)
		logger.error(msg)
		raise exceptions.UserException(msg)
	else:
		# For any of the allowed algorithms, check if the digest matches
		# the expected digest.
		for algorithm in settings.ALLOWED_DIGEST_ALGORITHMS:
			valid_digest = _to_bytes(_b64_digest(nonce, created, password,
				algorithm = algorithm.lower()))

			if ((valid_digest == encoded_digest) or
				(valid_digest == _to_bytes(encoded_digest))):
				return username
 
		# Check all of the prohibited algorithms - if the received digest matches
		# that of a prohibited algorithm, then an error is raised saying that
		# the algorithm is prohibited.
		for algorithm in settings.PROHIBITED_DIGEST_ALGORITHMS:
			valid_digest = _b64_digest(nonce, created, password,
				algorithm = algorithm.lower())

			if valid_digest == encoded_digest:
				msg = '''Prohibited algorithm {} used for digest. Please use one of the
following algorithms: {!r}.
Note that these algorithms are prohibited: {!r}.'''.format(algorithm,
	settings.ALLOWED_DIGEST_ALGORITHMS, settings.PROHIBITED_DIGEST_ALGORITHMS)
				logger.info(msg)
				raise exceptions.AlgorithmProhibited(msg)

	return None

### Internal Methods

def _parse_token(token):
	'''
	Parse the token to extract the parameters.

	:param token: token to parse
	:type token: str

	:return: tuple of (Username, PasswordDigest, Nonce, Created)
	:rtype: tuple

	:raises ~.exceptions.InvalidToken: invalid token
	'''
	try:
		key_values = {match.group('key'): match.group('value')
			for match in _TOKEN_RE.finditer(token)}
	except (AttributeError, StopIteration):
		key_values = {}

	missing_params = []
	for param in ('Username', 'PasswordDigest', 'Nonce', 'Created'):
		if param not in key_values:
			missing_params.append(param)

	if missing_params:
		msg = 'Token {} missing parameters: {!r}.'.format(token, missing_params)
		logger.info(msg)

		raise exceptions.InvalidToken(msg)

	return (key_values['Username'], key_values['PasswordDigest'],
		key_values['Nonce'], key_values['Created'])

def _parse_timestamp(timestamp):
	'''
	Parse a timestamp. Attempts the formats specified in the settings in the
	order given.

	:param timestamp: timestamp to parse
	:rtype timestamp: str

	:return: parsed timestamp
	:rtype: datetime.datetime

	:raises exceptions.InvalidTimestamp: invalid timestamp format
	'''
	for fmt in settings.TIMESTAMP_FORMATS:
		try:
			return datetime.datetime.strptime(timestamp, fmt)
		except ValueError:
			# If the format ends with a '%z', it expects a UTC offset. However,
			# Python versions under 3.2 do not support the %z format. So, the offset
			# is parsed manually and the rest of the timestamp is passed on to the
			# original strptime parser.
			if fmt.endswith('%z'):
				match = _TS_OFFSET_RE.match(timestamp)
				if not match: continue

				try:
					# Parse the base part of the string and the offset (with direction).
					base = datetime.datetime.strptime(match.group('timestamp'), fmt[:-2])
					direction = -1 if match.group('direction') == '-' else 1
					offset = datetime.timedelta(hours = int(match.group('hour')),
						minutes = int(match.group('minute')))

					return base + (direction * offset)

				except (ValueError, IndexError):
					continue

			else: continue
		
	msg = 'Invalid timestamp {}, expected one format from {!r}.'.format(
		timestamp, settings.TIMESTAMP_FORMATS)
	logger.info(msg)
	raise exceptions.InvalidTimestamp(msg)

def _random_string(length = None, allowed_chars = None):
	'''
	Generate a random string of the given length.

	:param length: length of the string (defaults to settings.NONCE_LENGTH)
	:rtype length: int

	:param allowed_chars: characters to allow in string
	:rtype allowed_chars: str

	:return: generated string
	:rtype: str
	'''
	if allowed_chars is None:
		try:
			allowed_chars = string.letters
		except AttributeError:
			allowed_chars = string.ascii_letters

	if length is None:
		length = settings.NONCE_LENGTH

	s = ''.join(random.choice(allowed_chars) for _ in range(length))
	return s

def _b64_digest(*args, **kwargs):
	'''
	Perform a digest on the arguments using the specified algorithm in the
	settings.

	:param args: arguments to digest
	:rtype args: `iter` of `str` or `bytes`
	'''
	algorithm = kwargs.get('algorithm')
	digest_algorithm = _get_digest_algorithm(algorithm)
	args_str = ''.join(map(_from_bytes, args))

	return base64.b64encode(digest_algorithm(_to_bytes(args_str)).digest())

def _get_digest_algorithm(name = None):
	'''
	Get the digest algorithm to use based on the settings.

	:param name: name of algorithm to use
	:type name: str

	:return: digest algorithm
	:rtype: class
	'''
	if name:
		possible_algorithms = [name.lower()]
	else:
		possible_algorithms = filter(lambda a: a in hashlib.algorithms_available,
			(map(str.lower, settings.ALLOWED_DIGEST_ALGORITHMS)))

	for algo_name in possible_algorithms:
		if hasattr(hashlib, algo_name):
			return getattr(hashlib, algo_name)

	logger.error('No algorithm from %r found in hashlib %r',
		settings.ALLOWED_DIGEST_ALGORITHMS, hashlib.algorithms_available)
	raise exceptions.AlgorithmNotSupported('No suitable algorithm found.')

def _get_nonce_store():
	'''
	Get the nonce store from the given setting.
	'''
	store_name = settings.NONCE_STORE

	if store_name in _STORES:
		return _STORES[store_name]

	store = pydoc.locate(store_name)(*settings.NONCE_STORE_ARGS)

	_STORES[store_name] = store

	return store

def _to_bytes(s):
	'''
	Convert a string to bytes if it is not already bytes.

	:param s: string to convert
	:type s: `str` or `bytes`

	:return: encoded string as binary data
	:rtype: bytes
	'''
	if not isinstance(s, bytes):
		return s.encode('utf-8')

	return s

def _from_bytes(b):
	'''
	Convert a string from bytes if it is not already a string.

	:param b: string to convert
	:type b: `str` or `bytes`

	:return: decoded string 
	:rtype: str
	'''
	if isinstance(b, bytes):
		return b.decode('utf-8')

	return b

def _django_header(header):
	'''
	Convert a header name to a Django header name.

	For example, the X-Header argument is transformed into HTTP_X_HEADER.

	:param header: header to transform
	:type header: str

	:return: transformed header
	:rtype: str
	'''
	return 'HTTP_{}'.format(header.replace('-', '_'))



# wsse/server/django/store.py
# coding=utf-8
# py-wsse
# Authors: Rushy Panchal, Naphat Sanguansin, Adam Libresco, Jérémie Lumbroso
# Date: August 30th, 2016
# Description: Store implementation for the Django server.

import datetime

from .models import WSSEEvent
from ... import settings

class DjangoNonceStore(object):
	'''
	Nonce store for Django ORM.
	'''
	def add_nonce(self, nonce, timestamp = None):
		'''
		Add a nonce to the store.

		:param nonce: nonce to store
		:type nonce: str

		:param timestamp: timestamp the nonce was generated at
		:type timestamp: datetime.datetime
		'''
		kwargs = {'nonce': nonce}
		if timestamp:
			kwargs['timestamp'] = timestamp

		WSSEEvent.objects.create(**kwargs)

	def has_nonce(self, nonce):
		'''
		Check if a nonce is in the store.

		:param nonce: nonce to check
		:type nonce: str
		'''
		self.clean_expired_nonces()

		return WSSEEvent.objects.filter(nonce = nonce).exists()

	def clean_expired_nonces(self):
		'''
		Clean any expired nonces from the database.
		'''
		now = datetime.datetime.utcnow().replace(microsecond = 0)
		exp_time = now - datetime.timedelta(seconds = settings.TIMESTAMP_DURATION)

		WSSEEvent.objects.filter(timestamp__lt = exp_time).delete()



# wsse/__init__.py
# coding=utf-8
# py-wsse
# Authors: Rushy Panchal, Naphat Sanguansin, Adam Libresco, Jérémie Lumbroso
# Date: August 30th, 2016

import logging

from . import settings

# If a handler is not already present, add a basic null handler to suppress
# errors.
logger = logging.getLogger(settings.LOGGER_NAME)

if not logger.handlers:
	logger.addHandler(logging.NullHandler())



# wsse/server/default/store.py
# coding=utf-8
# py-wsse
# Authors: Rushy Panchal, Naphat Sanguansin, Adam Libresco, Jérémie Lumbroso
# Date: August 30th, 2016
# Description: Store implementation for default server.

import sqlite3
import contextlib
import datetime

from ... import settings

class SQLiteNonceStore(object):
	'''
	Default store - supports adding, validating, and purging nonces.

	:param connection_url: URL to connect to
	:type connection_url: str

	:param table: name of the table to store nonces in
	:type table: str
	'''
	def __init__(self, connection_url, table = 'nonce_store'):
		self.conn_url = connection_url
		self.table_name = table
		self.db = None
		self.open()

	def open(self):
		'''
		Open the store.
		'''
		if self.db is None:
			self.db = sqlite3.connect(self.conn_url)
			self.__setup()

	def close(self):
		'''
		Close the store.
		'''
		if self.db is not None:
			self.db.commit()
			self.db.close()
			self.db = None

	@contextlib.contextmanager
	def __transaction(self):
		'''Perform a transaction on the database.'''
		cursor = self.db.cursor()
		yield cursor
		self.db.commit()

	def __setup(self):
		'''Setup the database.'''
		with self.__transaction() as cursor:
			cursor.execute('''CREATE TABLE IF NOT EXISTS
				{table} (id INTEGER PRIMARY KEY,
										 nonce VARCHAR(255) NOT NULL,
										 ts DATETIME DEFAULT CURRENT_TIMESTAMP,
										 UNIQUE(nonce))
				'''.format(table = self.table_name))
			cursor.execute('''CREATE INDEX IF NOT EXISTS idx_nonce ON
				{table}(nonce)'''.format(table = self.table_name))
			cursor.execute('''CREATE INDEX IF NOT EXISTS idx_ts ON
				{table}(ts)'''.format(table = self.table_name))

	def _clear(self):
		'''
		Clear the nonce store.
		'''
		with self.__transaction() as cursor:
			cursor.execute('DELETE FROM {table} WHERE 1=1'.format(
				table = self.table_name))

	def add_nonce(self, nonce, timestamp = None):
		'''
		Add a nonce to the store.

		:param nonce: nonce to store
		:type nonce: str

		:param timestamp: timestamp the nonce was generated at
		:type timestamp: datetime.datetime
		'''
		if not timestamp:
			timestamp = datetime.datetime.utcnow()

		with self.__transaction() as cursor:
			query = 'INSERT INTO {table} (nonce, ts) VALUES (:nonce, :ts)'.format(
				table = self.table_name)
			cursor.execute(query, {'nonce': nonce, 'ts': timestamp})

	def has_nonce(self, nonce):
		'''
		Check if a nonce is in the store.

		:param nonce: nonce to check
		:type nonce: str
		'''
		self.clean_expired_nonces()

		with self.__transaction() as cursor:
			# Check if the nonce already exists.
			select_query = 'SELECT id FROM {table} WHERE nonce = :nonce'.format(
				table = self.table_name)
			cursor.execute(select_query, {'nonce': nonce})
			return bool(cursor.fetchone())

	def clean_expired_nonces(self):
		'''
		Clean any expired nonces from the database.
		'''
		now = datetime.datetime.utcnow()
		exp_time = now - datetime.timedelta(seconds = settings.TIMESTAMP_DURATION)

		with self.__transaction() as cursor:
			delete_query = 'DELETE FROM {table} WHERE ts < :expired_time'.format(
				table = self.table_name)
			cursor.execute(delete_query, {'expired_time': exp_time})



# setup.py
# coding=utf-8
# py-wsse
# Authors: Rushy Panchal, Naphat Sanguansin, Adam Libresco, Jérémie Lumbroso
# Date: August 31st, 2016
# Description: Setuptools configuration.

from setuptools import setup

setup(
	name = 'pywsse',
	packages = [
		'wsse',
		'wsse.server',
		'wsse.server.django',
		'wsse.server.drf',
		'wsse.server.default',
		'wsse.client',
		'wsse.client.requests',
		],
	version = '0.1.1',
	description = 'WSSE Authentication for various server and client backends.',
	author = 'Rushy Panchal',
	author_email = 'rushy.panchal@princeton.edu',
	url = 'https://github.com/PrincetonUniversity/py-wsse',
	keywords = ['rest', 'authentication', 'wsse'],
	license = 'LGPLv3',
	classifiers = [
		'Programming Language :: Python :: 2',
		'Programming Language :: Python :: 2.7',
		'Programming Language :: Python :: 3',
		'Programming Language :: Python :: 3.1',
		'Programming Language :: Python :: 3.2',
		'Programming Language :: Python :: 3.3',
		'Programming Language :: Python :: 3.4',
		'Programming Language :: Python :: 3.5',
		'Programming Language :: Python :: 3.6',
		'Topic :: Internet :: WWW/HTTP :: WSGI :: Middleware',
		'Topic :: Software Development :: Libraries :: Python Modules',
		'Intended Audience :: Developers',
		'Operating System :: OS Independent',
		'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
		],
	tests_require = [
		'nose',
		'mock>=2.0.0',
		'django>=1.9',
		'djangorestframework>=3.4.6',
		'requests>=2.11.1',
		],
	test_suite = 'nose.collector',
)



# wsse/server/django/app.py
# coding=utf-8
# py-wsse
# Authors: Rushy Panchal, Naphat Sanguansin, Adam Libresco, Jérémie Lumbroso
# Date: September 2nd, 2016
# Description: App configuration for Django.

from django.apps import AppConfig

class WsseConfig(AppConfig):
	name = 'wsse'
